// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>

#include "GraphicsContext.hh"

GraphicsContext::GraphicsContext(GdkWindow* window, GdkColormap* colormap,
				 const GraphicsContextValues* values,
				 unsigned mask)
{
  assert(window != NULL);
  assert(colormap != NULL);

  gdk_window = window;

  gdk_gc = gdk_gc_new(window);
  assert(gdk_gc != NULL);

  gdk_colormap = colormap;
}

GraphicsContext::GraphicsContext(GdkGC* gc)
{
  assert(gc != NULL);

  gdk_window = NULLPTR;
  gdk_gc_ref(gc);
  gdk_gc = gc;

  gdk_colormap = NULLPTR;
}

GraphicsContext::GraphicsContext()
{
  gdk_window = NULLPTR;
  gdk_gc = NULLPTR;
}

GraphicsContext::~GraphicsContext()
{
  gdk_gc_unref(gdk_gc);
}

GraphicsContext* GraphicsContext::Clone() const
{
  assert(gdk_window != NULL);

  GraphicsContext* gc = new GraphicsContext();

  *gc = *this;

  gc->gdk_gc = gdk_gc_new(gdk_window);
  assert(gc->gdk_gc != NULL);

  return gc;
}

void GraphicsContext::SetLineStyle(LineStyle style, unsigned width)
{
  GdkLineStyle gdk_line_style =
    (style == LINE_STYLE_SOLID) ? GDK_LINE_SOLID : GDK_LINE_ON_OFF_DASH;
  
  gdk_gc_set_line_attributes(gdk_gc, width, gdk_line_style, GDK_CAP_BUTT, GDK_JOIN_MITER);
}

void GraphicsContext::SetColor(RGBValue c)
{
  assert(gdk_colormap != NULL);

  GdkColor gdk_color;
  //gdk_colormap_free_colors(gdk_colormap, &gdk_color, 1);

  // gdk colors use 16 bit component values
  gdk_color.red =   GETRED(c) * 256;
  gdk_color.green = GETGREEN(c) * 256;
  gdk_color.blue =  GETBLUE(c) * 256;

  gdk_colormap_alloc_color(gdk_colormap, &gdk_color, FALSE, TRUE);

  gdk_gc_set_foreground(gdk_gc, &gdk_color);  
}

void GraphicsContext::SetBackground(RGBValue c)
{
  assert(gdk_colormap != NULL);

  GdkColor gdk_background;
  //gdk_colormap_free_colors(gdk_colormap, &gdk_background, 1);

  gdk_background.red =   GETRED(c) * 256;
  gdk_background.green = GETGREEN(c) * 256;
  gdk_background.blue =  GETBLUE(c) * 256;

  gdk_colormap_alloc_color(gdk_colormap, &gdk_background, FALSE, TRUE);

  gdk_gc_set_background(gdk_gc, &gdk_background);
}

void GraphicsContext::SetClipRectangle(scaled x, scaled y, scaled width, scaled height)
{
  GdkRectangle gdk_clip;
  gdk_clip.x = sp2ipx(x);
  gdk_clip.y = sp2ipx(y);
  gdk_clip.width = sp2ipx(width);
  gdk_clip.height = sp2ipx(height);

  gdk_gc_set_clip_origin(gdk_gc, 0, 0);
  gdk_gc_set_clip_rectangle(gdk_gc, &gdk_clip);
}

void GraphicsContext::SetClipRectangle(const Rectangle* clip)
{
  if (clip != NULL) SetClipRectangle(clip->x, clip->y, clip->width, clip->height);
  else gdk_gc_set_clip_rectangle(gdk_gc, NULLPTR);
}

void GraphicsContext::GetLineStyle(LineStyle* style, unsigned* width) const
{
  assert(NOT_IMPLEMENTED);
  //if (style != NULL) *style = lineStyle;
  //if (width != NULL) *width = lineWidth;
}

