// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <iostream>
#include <iomanip.h>

#include "globals.hh"
#include "CharMapper.hh"
#include "MathMLElement.hh"
#include "MathMLCharNode.hh"

MathMLCharNode::MathMLCharNode(Char c)
{
  ch = c;
  fChar = NULL;

  BreakId bid;
  int s;

  if (isNonMarkingChar(c, &s, &bid) > 0) {
    isNonMarking = 1;
    SetSpacing(s);
    SetBreakability(bid);
  }
}

MathMLCharNode::~MathMLCharNode()
{
  delete fChar;
}

void
MathMLCharNode::Dump() const
{
  PrintIndentation();
  if (isPlain(ch)) cout << static_cast<char>(ch) << endl;
  else if (isUnicode16(ch)) cout << "&#x" << setw(4) << setfill('0') << hex << ch << ';' << endl;
  else cout << "&#x" << setw(8) << setfill('0') << hex << ch << ';' << endl;
  cout << dec;
}

void
MathMLCharNode::Fontify(const FontAttributes& fontAttributes)
{
  if (!isNonMarking) fChar = charMapper.FontifyChar(fontAttributes, ch);
}

void
MathMLCharNode::DoLayout()
{
  if (fChar != NULL) fChar->GetBoundingBox(box);
  else box.Zero();
}

void
MathMLCharNode::Render(const DrawingArea& area)
{
  if (!HasDirtyChildren()) return;

  if (fChar != NULL) {
    assert(GetParent() != NULL);
    assert(GetParent()->IsToken());
    area.DrawNativeChar(GetParent()->GetGraphicsContext(), fChar->font, GetX(), GetY(), fChar->nch);
  }

  ResetDirty();
}

bool
MathMLCharNode::IsChar() const
{
  return true;
}

bool
MathMLCharNode::HasDecimalPoint() const
{
  return GetChar() == '.';
}

scaled
MathMLCharNode::GetDecimalPointEdge() const
{
  return GetX() + ((GetChar() == '.') ? 0 : box.width);
}

