// Copyright (C) 2000, Luca Padovani <luca.padovani@cs.unibo.it>.
// 
// This file is part of GtkMathView, a Gtk widget for MathML.
// 
// GtkMathView is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// GtkMathView is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with GtkMathView; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// For details, see the GtkMathView World-Wide-Web page,
// http://cs.unibo.it/~lpadovan/mml-widget, or send a mail to
// <luca.padovani@cs.unibo.it>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <assert.h>
#include <iostream>
#include <iomanip.h>
#include <string.h>

#include "globals.hh"
#include "FontManager.hh"
#include "MathMLElement.hh"
#include "MathMLGlyphNode.hh"

MathMLGlyphNode::MathMLGlyphNode(const char* n, const char* f, NativeChar nc)
{
  assert(n != NULL);
  assert(f != NULL);
  assert(strlen(n) > 0);
  assert(strlen(f) > 0);

  name = new char[strlen(n)];
  fontFamily = new char[strlen(f)];

  strcpy(name, n);
  strcpy(fontFamily, f);

  nch = nc;

  font = NULL;
}

MathMLGlyphNode::~MathMLGlyphNode()
{
  delete [] name;
  delete [] fontFamily;
}

void
MathMLGlyphNode::Dump() const
{
  PrintIndentation();
  cout << "<mglyph alt=\"" << name << "\" ";
  cout << "fontfamily=\"" << fontFamily << "\" ";
  cout << "index=\"" << nch << "\"/>" << endl;
}

void
MathMLGlyphNode::Fontify(const FontAttributes& fontAttributes)
{
  FontAttributes glyphAttributes = fontAttributes;
  glyphAttributes.mask |= FONT_HAS_FAMILY;
  glyphAttributes.family = fontFamily;

  font = fontManager.GetFont(glyphAttributes);
}

void
MathMLGlyphNode::DoLayout()
{
  if (font != NULL) font->CharBox(nch, box);
  else box.Zero();
}

void
MathMLGlyphNode::Render(const DrawingArea& area)
{
  if (!HasDirtyChildren()) return;

  if (font != NULL) {
    assert(GetParent() != NULL);
    assert(GetParent()->IsToken());
    area.DrawNativeChar(GetParent()->GetGraphicsContext(), font, GetX(), GetY(), nch);
  }

  ResetDirty();
}
